@@ -109,8 +109,9 @@ class Tiingo(DataVendor):
         if categories is None:
             self.categories = ["crypto", "fx", "eqty"]
         if api_key is None:
-            raise TypeError("Set your api key. We recommend setting your api key in environment variables as"
-                            "'TIINGO_API_KEY', will allow DataCredentials to automatically load it.")
+            raise TypeError("Set your Tiingo api key in environment variables as 'TIINGO_API_KEY' or "
+                            "add it as an argument when instantiating the class. To get an api key, visit: "
+                            "https://www.tiingo.com/")
         if exchanges is None:
             self.exchanges = self.get_exchanges_info()
         if assets is None:
@@ -439,6 +440,7 @@ class Tiingo(DataVendor):
 
         # fx
         elif data_type == 'fx':
+            # url = f"https://api.tiingo.com/tiingo/fx/{ticker}/prices"
             url = self.base_url + f"fx/prices"
             headers = {
                 "Content-Type": "application/json",
@@ -557,6 +559,7 @@ class Tiingo(DataVendor):
             for mkt, ticker in zip(tg_data_req['mkts'], data_req.tickers):
                 try:
                     df0 = self.get_tidy_data(data_req, data_type, mkt)
+                    print(df0)
                 except Exception:
                     logging.info(f"Failed to get {data_type} data for {ticker} after many attempts.")
                 else:
