@@ -1,5 +1,4 @@
 from typing import Optional
-
 import pandas as pd
 
 from cryptodatapy.extract.data_vendors.coinmetrics_api import CoinMetrics
@@ -11,6 +10,7 @@ from cryptodatapy.extract.libraries.ccxt_api import CCXT
 from cryptodatapy.extract.libraries.dbnomics_api import DBnomics
 from cryptodatapy.extract.libraries.investpy_api import InvestPy
 from cryptodatapy.extract.libraries.pandasdr_api import PandasDataReader
+from cryptodatapy.extract.web.aqr import AQR
 
 
 class GetData:
@@ -96,9 +96,11 @@ class GetData:
             "fred": PandasDataReader,
             "av-daily": PandasDataReader,
             "av-forex-daily": PandasDataReader,
+            "famafrench": PandasDataReader,
+            "aqr": AQR
         }
 
-        # available properties and methods
+        # available attr and methods
         valid_attr = [
             "source_type",
             "categories",
@@ -203,6 +205,8 @@ class GetData:
             "fred": PandasDataReader,
             "av-daily": PandasDataReader,
             "av-forex-daily": PandasDataReader,
+            "famafrench": PandasDataReader,
+            "aqr": AQR
         }
 
         # data source
@@ -216,3 +220,71 @@ class GetData:
         df = getattr(ds, method)(self.data_req)
 
         return df
+
+    async def get_series_async(self, method: str = "get_data_async") -> pd.DataFrame:
+        """
+        Get requested data.
+
+        Parameters
+        ----------
+        method: str, default 'get_data'
+            Gets the specified method from the data source object.
+
+        Returns
+        -------
+        df: pd.DataFrame - MultiIndex
+            DataFrame with DatetimeIndex (level 0), ticker (level 1), and field (cols) values.
+
+        Examples
+        --------
+        >>> data_req = DataRequest(source='ccxt', tickers=['btc', 'eth'], fields=['open', 'high', 'low', 'close',
+                                   'volume'], freq='d', exch='ftx', start_date='2017-01-01')
+        >>> GetData(data_req).get_series()
+                                open        high        low         close       volume
+        date	    ticker
+        2020-03-28	BTC	        6243.25	    6298.5	    6028.0	    6237.5	    3888.9424
+                    ETH	        128.995	    133.0	    125.11	    131.04	    1751.65972
+        2020-03-29	BTC	        6233.5	    6262.5	    5869.5	    5876.5	    114076.5831
+                    ETH	        130.98	    131.84	    123.81	    124.33	    138449.60906
+        2020-03-30	BTC	        5876.0	    6609.0	    5856.0	    6396.5	    224231.1718
+
+
+        >>> data_req = DataRequest(source='glassnode', tickers=['btc', 'eth'],
+                                   fields=['add_act', 'tx_count', 'issuance'], freq='d', start_date='2016-01-01')
+        >>> GetData(data_req).get_series()
+                                add_act     tx_count    issuance
+        date        ticker
+        2016-01-01	BTC	        316489	    123957	    0.085386
+                    ETH	        2350	    8232	    0.133048
+        2016-01-02	BTC	        419389	    148893	    0.09197
+                    ETH	        2410	    9164	    0.140147
+        2016-01-03	BTC	        394047	    142463	    0.091947
+        """
+        # data source objects
+        data_source_dict = {
+            "cryptocompare": CryptoCompare,
+            "coinmetrics": CoinMetrics,
+            "ccxt": CCXT,
+            "glassnode": Glassnode,
+            "tiingo": Tiingo,
+            "investpy": InvestPy,
+            "dbnomics": DBnomics,
+            "yahoo": PandasDataReader,
+            "fred": PandasDataReader,
+            "av-daily": PandasDataReader,
+            "av-forex-daily": PandasDataReader,
+            "famafrench": PandasDataReader,
+            "aqr": AQR
+        }
+
+        # data source
+        ds = data_source_dict[self.data_req.source]
+        # instantiate ds obj
+        if self.api_key is not None:
+            ds = ds(api_key=self.api_key)
+        else:
+            ds = ds()
+        # get data
+        df = await getattr(ds, method)(self.data_req)
+
+        return df
