@@ -2,7 +2,6 @@ from __future__ import annotations
 from typing import Union, Dict, List, Optional, Any
 from importlib import resources
 
-
 import pandas as pd
 
 from cryptodatapy.extract.datarequest import DataRequest
@@ -470,8 +469,7 @@ class WrangleData:
     """
     Wrangles time series data responses from various APIs into tidy data format.
     """
-
-    def __init__(self, data_req: DataRequest, data_resp: Union[Dict[str, Any], pd.DataFrame]):
+    def __init__(self, data_req: DataRequest, data_resp: Union[Dict[str, pd.DataFrame], pd.DataFrame]):
         """
         Constructor
 
@@ -479,12 +477,13 @@ class WrangleData:
         ----------
         data_req: DataRequest
             Data request object with parameter values.
-        data_resp: pd.DataFrame
-            Dataframe containing data response.
+        data_resp: dictionary or pd.DataFrame
+            Dictionary with Dataframes or DataFrame containing data response.
 
         """
         self.data_req = data_req
         self.data_resp = data_resp
+        self.tidy_data = pd.DataFrame()
 
     def cryptocompare(self) -> pd.DataFrame:
         """
@@ -608,7 +607,6 @@ class WrangleData:
             self.data_resp.rename(columns={'t': 'date', 'o': 'open', 'h': 'high', 'c': 'close', 'l': 'low'},
                                   inplace=True)
             self.data_resp = self.data_resp.loc[:, ['date', 'open', 'high', 'low', 'close']]
-
         # convert to datetime
         self.data_resp['date'] = pd.to_datetime(self.data_resp['date'], unit='s')
         # set index
@@ -669,73 +667,6 @@ class WrangleData:
 
         return self.data_resp
 
-    def dbnomics(self) -> pd.DataFrame:
-        """
-        Wrangles DBnomics data response to dataframe with tidy data format.
-
-        Returns
-        -------
-        pd.DataFrame
-            Wrangled dataframe into tidy data format.
-
-        """
-        # df = self.convert_cols().convert_fields_to_lib().convert_to_datetime().set_idx().resample_freq(). \
-        #     filter_dates().type_conversion().remove_bad_data().data_resp
-        # convert fields to lib
-        self.convert_fields_to_lib(data_source='dbnomics')
-        # convert to datetime
-        self.data_resp['date'] = pd.to_datetime(self.data_resp['date'])
-        # set index
-        self.data_resp = self.data_resp.set_index('date').sort_index()
-        # resample
-        self.data_resp = self.data_resp.resample(self.data_req.freq).last().ffill()
-        # filte dates
-        self.filter_dates()
-        # type conversion
-        self.data_resp = self.data_resp.apply(pd.to_numeric, errors='coerce').convert_dtypes()
-        # remove bad data
-        self.data_resp = self.data_resp[self.data_resp != 0]  # 0 values
-        self.data_resp = self.data_resp[~self.data_resp.index.duplicated()]  # duplicate rows
-        self.data_resp = self.data_resp.dropna(how='all').dropna(how='all', axis=1)  # entire row or col NaNs
-
-        return self.data_resp
-
-    def ccxt(self) -> pd.DataFrame:
-        """
-        Wrangles CCXT data response to dataframe with tidy data format.
-
-        Returns
-        -------
-        pd.DataFrame
-            Wrangled dataframe into tidy data format.
-
-        """
-
-        # df = self.convert_cols().convert_fields_to_lib().convert_to_datetime().set_idx().resample_freq(). \
-        #     type_conversion().remove_bad_data().data_resp
-        # convert fields to lib
-        self.convert_fields_to_lib(data_source='ccxt')
-        # convert to datetime
-        if 'close' in self.data_resp.columns:
-            self.data_resp['date'] = pd.to_datetime(self.data_resp.date, unit='ms')
-        elif 'funding_rate' in self.data_resp.columns:
-            self.data_resp['date'] = pd.to_datetime(self.data_resp.set_index('date').index). \
-                floor('S').tz_localize(None)
-        # set index
-        self.data_resp = self.data_resp.set_index('date').sort_index()
-        # resample
-        if 'funding_rate' in self.data_resp.columns and self.data_req.freq in ['d', 'w', 'm', 'q', 'y']:
-            self.data_resp = (self.data_resp.funding_rate + 1).cumprod().resample(self.data_req.freq).last(). \
-                diff().to_frame()
-        # type conversion
-        self.data_resp = self.data_resp.apply(pd.to_numeric, errors='coerce').convert_dtypes()
-        # remove bad data
-        self.data_resp = self.data_resp[self.data_resp != 0]  # 0 values
-        self.data_resp = self.data_resp[~self.data_resp.index.duplicated()]  # duplicate rows
-        self.data_resp = self.data_resp.dropna(how='all').dropna(how='all', axis=1)  # entire row or col NaNs
-
-        return self.data_resp
-
     def investpy(self) -> pd.DataFrame:
         """
         Wrangles InvestPy data response to dataframe with tidy data format.
@@ -747,29 +678,21 @@ class WrangleData:
 
         """
         # convert cols
-        if self.data_req.cat != 'macro':
-            # reset index
-            self.data_resp = self.data_resp.reset_index()
-        else:
-            # parse date and time to create datetime
-            self.data_resp.time.replace('Tentative', '23:55', inplace=True)
-            self.data_resp['date'] = pd.to_datetime(self.data_resp.date + self.data_resp.time,
-                                                    format="%d/%m/%Y%H:%M")
+        # parse date and time to create datetime
+        self.data_resp.time.replace('Tentative', '23:55', inplace=True)
+        self.data_resp['date'] = pd.to_datetime(self.data_resp.date + self.data_resp.time,
+                                                format="%d/%m/%Y%H:%M")
         # convert fields to lib
         self.convert_fields_to_lib(data_source='investpy')
         # set index
         self.data_resp = self.data_resp.set_index('date').sort_index()
-        # replace % and compute surprise
-        if self.data_req.cat == 'macro':
-            self.data_resp = self.data_resp.replace('%', '', regex=True).astype(float) / 100  # remove % str
-            # replace missing vals
-            self.data_resp.expected.fillna(self.data_resp.previous, inplace=True)
-            self.data_resp['surprise'] = self.data_resp.actual - self.data_resp.expected
+        # replace % and missing vals
+        self.data_resp = self.data_resp.replace('%', '', regex=True).astype(float) / 100  # remove % str
+        self.data_resp.expected.fillna(self.data_resp.previous, inplace=True)
+        # compute surprise
+        self.data_resp['surprise'] = self.data_resp.actual - self.data_resp.expected
         # resample freq
         self.data_resp = self.data_resp.resample(self.data_req.freq).last().ffill()
-        # extend index and fwd fill to current date
-        ext_idx = pd.date_range(self.data_resp.index.max(), pd.Timestamp.today()).union(self.data_resp.index)
-        self.data_resp = self.data_resp.reindex(ext_idx).ffill()
         # filter dates
         self.filter_dates()
         # type conversion
@@ -777,9 +700,167 @@ class WrangleData:
         # remove bad data
         self.data_resp = self.data_resp[~self.data_resp.index.duplicated()]  # duplicate rows
         self.data_resp = self.data_resp.dropna(how='all').dropna(how='all', axis=1)  # entire row or col NaNs
+        # keep only requested fields
+        self.data_resp = self.data_resp[self.data_req.fields]
 
         return self.data_resp
 
+    def dbnomics(self) -> pd.DataFrame:
+        """
+        Wrangles DBnomics data response to dataframe with tidy data format.
+
+        Returns
+        -------
+        pd.DataFrame
+            Wrangled dataframe into tidy data format.
+
+        """
+        # convert fields to lib
+        self.convert_fields_to_lib(data_source='dbnomics')
+
+        # convert to datetime
+        self.data_resp['date'] = pd.to_datetime(self.data_resp['date'])
+
+        # set index
+        self.data_resp = self.data_resp.set_index('date').sort_index()
+
+        # resample
+        self.data_resp = self.data_resp.resample(self.data_req.freq).last().ffill()
+
+        # filter dates
+        self.filter_dates()
+
+        # type conversion
+        self.data_resp = self.data_resp.apply(pd.to_numeric, errors='coerce').convert_dtypes()
+
+        # remove bad data
+        self.data_resp = self.data_resp[self.data_resp != 0]  # 0 values
+        self.data_resp = self.data_resp[~self.data_resp.index.duplicated()]  # duplicate rows
+        self.data_resp = self.data_resp.dropna(how='all').dropna(how='all', axis=1)  # entire row or col NaNs
+
+        return self.data_resp
+
+    def ccxt_ohlcv(self) -> pd.DataFrame:
+        """
+        Wrangles CCXT OHLCV data response to dataframe with tidy data format.
+
+        Returns
+        -------
+        pd.DataFrame
+            Dataframe with tidy data format.
+        """
+        # field cols
+        cols = ["date", "open", "high", "low", "close", "volume"]
+
+        # add tickers
+        for i in range(len(self.data_req.source_markets)):
+            df = pd.DataFrame(self.data_resp[i], columns=cols)
+            df['ticker'] = self.data_req.source_markets[i]
+            self.tidy_data = pd.concat([self.tidy_data, df])
+
+        # convert to datetime
+        self.tidy_data['date'] = pd.to_datetime(self.tidy_data['date'], unit='ms')
+
+        # set index
+        self.tidy_data = self.tidy_data.set_index(['date', 'ticker']).sort_index()
+
+        return self.tidy_data
+
+    def ccxt_funding_rates(self) -> pd.DataFrame:
+        """
+        Wrangles CCXT funding rates data response to dataframe with tidy data format.
+
+        Returns
+        -------
+        pd.DataFrame
+            Dataframe with tidy data format.
+        """
+        # add tickers
+        for i in range(len(self.data_req.source_markets)):
+            df = pd.DataFrame(self.data_resp[i])
+            self.tidy_data = pd.concat([self.tidy_data, df])
+        self.tidy_data = self.tidy_data[['symbol', 'fundingRate', 'datetime']]
+        self.data_resp = self.tidy_data
+
+        # convert to lib fields
+        self.convert_fields_to_lib(data_source='ccxt')
+        self.tidy_data = self.data_resp
+
+        # convert to datetime
+        self.tidy_data['date'] = pd.to_datetime(self.tidy_data.set_index('date').index).floor('s').tz_localize(None)
+
+        # set index
+        self.tidy_data = self.tidy_data.set_index(['date', 'ticker']).sort_index()
+
+        # resample
+        if self.data_req.freq in ['d', 'w', 'm', 'q', 'y']:
+            self.tidy_data = (
+                    (1 + self.tidy_data.funding_rate)
+                    .groupby('ticker')
+                    .resample('d', level='date')
+                    .prod() - 1
+            ).to_frame().swaplevel('ticker', 'date').sort_index()
+
+        return self.tidy_data
+
+    def ccxt_open_interest(self) -> pd.DataFrame:
+        """
+        Wrangles CCXT open interest data response to dataframe with tidy data format.
+
+        Returns
+        -------
+        pd.DataFrame
+            Dataframe with tidy data format.
+        """
+        # add tickers
+        for i in range(len(self.data_req.source_markets)):
+            df = pd.DataFrame(self.data_resp[i])
+            self.tidy_data = pd.concat([self.tidy_data, df])
+        self.tidy_data = self.tidy_data[['symbol', 'openInterestAmount', 'datetime']]
+        self.data_resp = self.tidy_data
+
+        # convert to lib fields
+        self.convert_fields_to_lib(data_source='ccxt')
+        self.tidy_data = self.data_resp
+
+        # convert to datetime
+        self.tidy_data['date'] = pd.to_datetime(self.tidy_data.set_index('date').index).floor('s').tz_localize(None)
+
+        # set index
+        self.tidy_data = self.tidy_data.set_index(['date', 'ticker']).sort_index()
+
+        return self.tidy_data
+
+    def ccxt(self, data_type: str) -> pd.DataFrame:
+        """
+        Wrangles CCXT data response to dataframe with tidy data format.
+
+        Returns
+        -------
+        pd.DataFrame
+            Wrangled dataframe into tidy data format.
+
+        """
+        if data_type == 'ohlcv':
+            self.tidy_data = self.ccxt_ohlcv()
+        elif data_type == 'funding_rates':
+            self.tidy_data = self.ccxt_funding_rates()
+        elif data_type == 'open_interest':
+            self.tidy_data = self.ccxt_open_interest()
+        else:
+            raise ValueError(f"Data type {data_type} not supported.")
+
+        # type conversion
+        self.tidy_data = self.tidy_data.apply(pd.to_numeric, errors='coerce').convert_dtypes()
+
+        # remove bad data
+        if data_type != 'funding_rates':
+            self.tidy_data = self.tidy_data[self.tidy_data != 0]  # 0 values
+        self.tidy_data = self.tidy_data[~self.tidy_data.index.duplicated()]  # duplicate rows
+        self.tidy_data = self.tidy_data.dropna(how='all').dropna(how='all', axis=1)  # entire row or col NaNs
+
+        return self.tidy_data
+
     def fred(self) -> pd.DataFrame:
         """
         Wrangles Fred data response to dataframe with tidy data format.
@@ -788,24 +869,29 @@ class WrangleData:
         -------
         pd.DataFrame
             Wrangled dataframe into tidy data format.
-
         """
-        # convert tickers to cryptodatapy format
+        # tickers
         self.data_resp.columns = self.data_req.tickers  # convert tickers to cryptodatapy format
+
         # resample to match end of reporting period, not beginning
         self.data_resp = self.data_resp.resample('d').last().ffill().resample(self.data_req.freq).last().stack(). \
             to_frame().reset_index()
+
         # convert cols
         if self.data_req.cat == 'macro':
             self.data_resp.columns = ['DATE', 'symbol', 'actual']
         else:
             self.data_resp.columns = ['DATE', 'symbol', 'close']
-        # convert fields to lib
+
+        # fields
         self.convert_fields_to_lib(data_source='fred')
-        # set index
+
+        # index
         self.data_resp.set_index(['date', 'ticker'], inplace=True)
+
         # type conversion
         self.data_resp = self.data_resp.apply(pd.to_numeric, errors='coerce').convert_dtypes()
+
         # remove bad data
         self.data_resp = self.data_resp[self.data_resp != 0]  # 0 values
         self.data_resp = self.data_resp[~self.data_resp.index.duplicated()]  # duplicate rows
@@ -821,31 +907,193 @@ class WrangleData:
         -------
         pd.DataFrame
             Wrangled dataframe into tidy data format.
-
         """
-        # TODO: resample data
-        # resample
-        # self._data_resp = self.data_resp.resample(data_req.freq).last()
-        #
-        # stack and reset index
-        self.data_resp = self.data_resp.stack().reset_index()
-        # remove name
+        # tickers
+        tickers_dict = {source_ticker: ticker for source_ticker, ticker in zip(self.data_req.source_tickers,
+                        self.data_req.tickers)}
+        self.data_resp = self.data_resp.stack(future_stack=True)
         self.data_resp.columns.name = None
-        # convert fields
+        self.data_resp.index = self.data_resp.index.set_levels(self.data_resp.index.levels[1].map(tickers_dict),
+                                                               level=1)
+        self.data_resp.reset_index(inplace=True)
+
+        #  fields
         self.convert_fields_to_lib(data_source='yahoo')
-        # convert to datetime
+
+        # index
         self.data_resp['date'] = pd.to_datetime(self.data_resp['date'])
-        # set index
         self.data_resp.set_index(['date', 'ticker'], inplace=True)
-        # re-order cols
-        self.data_resp = self.data_resp.loc[:, ['open', 'high', 'low', 'close', 'close_adj', 'volume']]
+
+        # resample
+        self.data_resp = self.data_resp.groupby('ticker').\
+            resample(self.data_req.freq, level='date').\
+            last().swaplevel('ticker', 'date').sort_index()
+
         # type conversion
         self.data_resp = self.data_resp.apply(pd.to_numeric, errors='coerce').convert_dtypes()
+
         # remove bad data
         self.data_resp = self.data_resp[self.data_resp != 0]  # 0 values
         self.data_resp = self.data_resp[~self.data_resp.index.duplicated()]  # duplicate rows
         self.data_resp = self.data_resp.dropna(how='all').dropna(how='all', axis=1)  # entire row or col NaNs
 
+        # keep only requested fields and sort index
+        self.data_resp = self.data_resp[self.data_req.fields].sort_index()
+
+        return self.data_resp
+
+    def famafrench(self) -> pd.DataFrame:
+        """
+        Wrangles Fama-French data response to dataframe with tidy data format.
+
+        Returns
+        -------
+        pd.DataFrame
+            Wrangled dataframe into tidy data format.
+
+        """
+        # ticker
+        ff_tickers_dict = {'RF': 'US_Rates_1M_RF',
+                           'Mkt-RF': 'US_Eqty_CSRP_ER',
+                           'HML': 'US_Eqty_Val',
+                           'SMB': 'US_Eqty_Size',
+                           'RMW': 'US_Eqty_Prof',
+                           'CMA': 'US_Eqty_Inv',
+                           'Mom': 'US_Eqty_Mom',
+                           'ST_Rev': 'US_Eqty_STRev'}
+
+        # remove white space from cols str
+        self.data_resp.columns = [col.strip() for col in self.data_resp.columns]
+        # keep cols in data req tickers
+        self.data_resp.columns = [ff_tickers_dict[col] for col in self.data_resp.columns
+                                  if col in ff_tickers_dict.keys()]  # keep
+        drop_cols = [col for col in self.data_resp.columns if col not in self.data_req.tickers]
+        self.data_resp.drop(columns=drop_cols, inplace=True)
+        self.data_resp = self.data_resp.loc[:, ~self.data_resp.columns.duplicated()]  # drop dup cols
+
+        # resample freq
+        self.data_resp = self.data_resp.resample(self.data_req.freq).sum()
+
+        # format index
+        self.data_resp.index.name = 'date'  # rename
+        self.data_resp = self.data_resp.stack().to_frame('er')
+        self.data_resp.index.names = ['date', 'ticker']
+
+        # type and conversion to decimals
+        self.data_resp = self.data_resp.apply(pd.to_numeric, errors='coerce').convert_dtypes() / 100
+
+        # remove bad data
+        self.data_resp = self.data_resp[self.data_resp != 0]  # 0 values
+        self.data_resp = self.data_resp[~self.data_resp.index.duplicated()]  # duplicate rows
+        self.data_resp = self.data_resp.dropna(how='all').dropna(how='all', axis=1)  # entire row or col NaNs
+
+        return self.data_resp
+
+    def wb(self) -> pd.DataFrame:
+        """
+        Wrangles World Bank data response to dataframe with tidy data format.
+
+        Returns
+        -------
+        pd.DataFrame
+            Wrangled dataframe into tidy data format.
+
+        """
+        # convert tickers to cryptodatapy format
+        with resources.path("cryptodatapy.conf", "tickers.csv") as f:
+            tickers_path = f
+        tickers_df = pd.read_csv(tickers_path, index_col=0, encoding="latin1")
+        self.data_resp = self.data_resp.stack().to_frame()  # stack df
+        # create list of tickers using tickers csv
+        tickers = []
+        for row in self.data_resp.iterrows():
+            tickers.append(
+                tickers_df[(tickers_df.country_name == row[0][0]) & (tickers_df.wb_id == row[0][2])].index[0])
+        self.data_resp['ticker'] = tickers
+        # convert fields
+        self.data_resp = self.data_resp.reset_index().rename(columns={0: 'actual', 'year': 'date'})
+        # convert date
+        self.data_resp.date = pd.to_datetime(self.data_resp.date) + pd.tseries.offsets.YearEnd()
+        # set index
+        self.data_resp = self.data_resp[['date', 'ticker', 'actual']].set_index(['date', 'ticker']).sort_index()
+        # drop tickers
+        drop_tickers = list(set(self.data_resp.index.get_level_values(1).to_list()) - set(self.data_req.tickers))
+        self.data_resp = self.data_resp.drop(drop_tickers, level=1)
+
+        return self.data_resp
+
+    # TODO: fix resample to quarterly
+    def aqr(self) -> pd.DataFrame:
+        """
+        Wrangles AQR data file to dataframe with tidy data format.
+
+        Returns
+        -------
+        pd.DataFrame
+            Wrangled dataframe into tidy data format.
+        """
+        # convert tickers to cryptodatapy format
+        tickers_dict = {
+            'US_Eqty_Val': 'USA',
+            'US_Eqty_Size': 'USA',
+            'US_Eqty_Mom': 'USA',
+            'US_Eqty_Qual': 'USA',
+            'US_Eqty_Beta': 'USA',
+            'WL_Eqty_Val': 'Global',
+            'WL_Eqty_Size': 'Global',
+            'WL_Eqty_Mom': 'Global',
+            'WL_Eqty_Qual': 'Global',
+            'WL_Eqty_Beta': 'Global',
+            'WL_Eqty_Fut_Val': 'Equity indices Value',
+            'WL_Eqty_Fut_Mom': 'Equity indices Momentum',
+            'WL_Eqty_Fut_Carry': 'Equity indices Carry',
+            'WL_Eqty_Fut_Beta': 'Equity indices Defensive',
+            'WL_Rates_Val': 'Fixed income Value',
+            'WL_Rates_Mom': 'Fixed income Momentum',
+            'WL_Rates_Carry': 'Fixed income Carry',
+            'WL_Rates_Beta': 'Fixed income Defensive',
+            'WL_Cmdty_Val': 'Commodities Value',
+            'WL_Cmdty_Mom': 'Commodities Momentum',
+            'WL_Cmdty_Carry': 'Commodities Carry',
+            'WL_FX_Val': 'Currencies Value',
+            'WL_FX_Mom': 'Currencies Momentum',
+            'WL_FX_Carry': 'Currencies Carry',
+            'WL_Eqty_Fut_Mom_TS': 'TSMOM^EQ',
+            'WL_Rates_Mom_TS': 'TSMOM^FI',
+            'WL_Comdty_Mom_TS': 'TSMOM^CM',
+            'WL_FX_Mom_TS': 'TSMOM^FX',
+            'Cmdty_ER': 'Excess return of equal-weight commodities portfolio',
+            'US_Credit_ER': 'CORP_XS',
+            'US_Rates_Long_ER': 'GOVT_XS',
+            'US_Rates_1M_RF': 'Risk Free Rate'
+        }
+        # empty df
+        df = pd.DataFrame()
+        # loop through dfs dict
+        for ticker in self.data_resp.keys():
+            # keep ticker col and rename col
+            df1 = self.data_resp[ticker][[tickers_dict[ticker]]]
+            df1.columns = [ticker]
+            # rename index
+            df1.index.name = 'date'
+            # resample
+            if self.data_req.freq != 'd':
+                df1 = df1.resample(self.data_req.freq).sum()
+            # concat to df
+            df = pd.concat([df, df1], join='outer', axis=1)
+        # filter dates
+        self.data_resp = df
+        self.filter_dates()
+        # stack df
+        self.data_resp = self.data_resp.stack().to_frame('er')
+        # create multi index
+        self.data_resp.index.names = ['date', 'ticker']
+        # type and conversion to decimals
+        self.data_resp = self.data_resp.apply(pd.to_numeric, errors='coerce').convert_dtypes()
+        # remove bad data
+        self.data_resp = self.data_resp[~self.data_resp.index.duplicated()]  # duplicate rows
+        self.data_resp = self.data_resp.dropna(how='all').dropna(how='all', axis=1)  # entire row or col NaNs
+
         return self.data_resp
 
     def convert_fields_to_lib(self, data_source: str) -> WrangleData:
@@ -872,9 +1120,9 @@ class WrangleData:
 
         # loop through data resp cols
         for col in self.data_resp.columns:
-            if self.data_req.source_fields is not None and col in self.data_req.source_fields:
-                pass
-            elif col in fields_list or col.title() in fields_list or col.lower() in fields_list:
+            # if self.data_req.source_fields is not None and col in self.data_req.source_fields:
+            #     pass
+            if col in fields_list or col.title() in fields_list or col.lower() in fields_list:
                 self.data_resp.rename(columns={col: fields_df[(fields_df[str(data_source) + '_id']
                                       == col.title()) |
                                     (fields_df[str(data_source) + '_id'] == col.lower()) |
